--File Salesmn2.ex replaces Salesman.ex
--Written by Arthur Adamson, Feb. 1997, euclid@horandata.net
--and submitted to Rob Craig's Euphoria home page.
 
--This is version 2 with improved messages and comments, improved scaling
--to fit screen, scaling not allowed to distort trip distance reported,
--and faster operation.
 
--This is an approximate solution to the famous traveling-salesman problem.
--A salesman needs to visit each of a number of cities and return home. He
--would like to use the shortest path. Finding the answer to this
--seemingly simple problem requires an immense amount of computation
--to find the best solution. Since there are n-1 factorial possible paths,
--the search-all approach is impractical unless n is small.
 
--The approximate method used here greatly reduces the computation required
--but gives no guarantee of finding the best solution, only a pretty good one.
 
--I use the technique described in Scientific American, Mar., 1997, with
--some shortcuts/mods/ omissions. I may restore some of the omissions later.
 
--It seems to work remarkably well on my Pentium 120 MHZ DOS 6.22.
--First I create a sequence of n cities and a random initial path (oldTrip).
--The cities are given random but bounded x and y coordinates. (For a specific
--problem, the desired real coordinates would be used.)
--A second n X n sequence contains the distances between the cities.
--Distance for oldTrip is calculated, then oldTrip is perturbed (newTrip). 
--The oldTrip is replaced by newTrip only when newTrip is shorter.
--Such iterations continue until the runLimit for number of iterations is
--reached.
 
--  The perturbations are,
--1. Random length and location path-segment removal and replacement at a
--random location.
--2. Random reversal of the removed segment before replacement
 
--Run the program. It will ask how many cities. Type in an integer. Watch the
--convergence toward a "good" or sometimes perfect answer.
 
--The output is a column of total path length, a column of the corresponding
--iteration number, and a plot of the final trip path. The plot occasionally
--has a missing side. This is due to cities too close to be seperated on a
--the plot. They are usually signalled by the failure to change color at 
--discontinuity...ei there is a near zero length at the corner.
--If there any "loops" in the output, it probably did not converge. 
--Run it again.
 
--The output also shows the number of cities and (numCities-1)! for
--comparison with the number of iterations, showing the immense reduction of 
--computations...but with the penalty of no certainty that the "right" answer
--is found.
 
--The last trip length shown is the "answer". The extra iterations beyond
--the last trip length shown did not find any better path.
 
--The max iterations (runLimit) is set at number of cities cubed. This 
--seems to be a good compromise for the number of cities I have tested, ie 
--5 to 150. If extreme accuracy is important for your problem, you may need
--to increase runLimit.
 
--At line 216 is a provision for inputting a circular grouping of cities,
--which you and I can use as a test to show that the best path is found at
--least for a circle. 
 
--At line 226, a group of lines for inputting a small, easily polygon is
--provided so the path length accuracy can be checked.
 
--The sequence cityCoords could be set to your specific problem if desired.
 
--All trials start with a "random" ordering of path.
 
include get.e
include graphics.e
include select.e
include math.e
--------------------------------------------------------------
--global vars
integer  numCities, runLimit, city1, city2, IL, SI, EI, II
sequence cityCoords, cityDistances, oldTrip, newTrip
constant GRAPHICS_MODE = 18 
----------------------------------------------------------------
function getExtremes(sequence seqIn)   --used to scale coords for drawing
				       --Called by drawIt()
atom  xMax, yMax, xMin, yMin
integer lengt
lengt = length(seqIn)
xMax = 0.0 yMax = 0.0 xMin = 0.0 yMin = 0.0
for i = 1 to lengt by 1 do
if seqIn[i][1] > xMax then xMax = seqIn[i][1] end if
if seqIn[i][2] > yMax then yMax = seqIn[i][2] end if
if seqIn[i][1] < xMin then xMin = seqIn[i][1] end if
if seqIn[i][2] < yMin then yMin = seqIn[i][2] end if
end for
return {xMax, yMax, xMin, yMin}
end function
----------------------------------------------------------------
function getn()   --Called by main to get the number of cities.
    object o
    atom a
    o = get(0)
    a = o[2]
    return  a
end function
 
-----------------------------------------------------------------
procedure drawIt()   
--Called by main to depict the city and path relative locations.
--Most of this deals with scaling and shifting to keep the plot on screen
 
atom xMax, yMax, xMin, yMin, xSpread, ySpread,xMean, yMean
atom xMult, yMult, mult
sequence extremes
integer color
extremes = repeat(0,numCities)
extremes =  getExtremes(cityCoords)
xMax = extremes[1]
yMax = extremes[2]
xMin = extremes[3]
yMin = extremes[4]
xSpread = xMax - xMin
ySpread = yMax - yMin
xMean = (xMax + xMin) / 2
yMean = (yMax + yMin) / 2
xMult = 400 / xSpread       --assumes 400 x 400  drawing area
yMult = 400 / ySpread       --assumes 400 x 400  drawing area
			--assumes 400 x 400  drawing area centered at 300
			--assumes 400 x 400  drawing area centered at 300
if xSpread > ySpread then mult = xMult else  mult = yMult end if 
cityCoords = cityCoords * mult
for i = 1 to numCities by 1 do
  cityCoords[i][1] = cityCoords[i][1] + 300 - mult * xMean
  cityCoords[i][2] = cityCoords[i][2] + 200 - mult * yMean
end for
for i = 1 to numCities-1 by 1 do
    city1 = oldTrip[i]
    city2 = oldTrip[i+1]
color = remainder(i,2) + 3  --this flips the red/green for alternate legs
draw_line(color,{cityCoords[city1], cityCoords[city2]})
end for
draw_line(14,{cityCoords[city2], cityCoords[oldTrip[1]]}) 
--Closure leg also has a different color (yellow)
end procedure
 
-----------------------------------------------------------------
--Use this for reversing the exchange segment when random requires
function reverse(sequence seqIn)     --Called from anneal
sequence seqOut 
integer lengt, lengtp1
lengt = length(seqIn)
lengtp1 = lengt + 1
seqOut = repeat(0,lengt)
    for i = 1 to lengt by 1 do  seqOut[lengtp1 - i] = seqIn[i] end for
    return   seqOut
end function
-----------------------------------------------------------------
--Use this to calculate the length of each path
function calcTripDist(sequence seqIn)
--Calculate the trip distance for oldTrip if oldTrip is seqIn
--Called once by anneal
atom dist
dist = 0.0
for i = 1 to numCities  by 1 do
    city1 = seqIn[i]
    if i < numCities then
	city2 = seqIn[i+1]
	dist = dist + cityDistances[city1][city2]
	else    --The closing return to home
	city2 = seqIn[1]
	dist = dist + cityDistances[city2][city1]
    end if
end for 
return dist
end function
----------------------------------------------------------
 
procedure loadDistances() 
--called by setup, initialize a table of distance for city pairs so the
--calculations do not have to be repeated again.
atom xD, yD 
cityDistances = repeat(repeat(0,numCities),numCities)
oldTrip = repeat(0,numCities)
newTrip = repeat(0,numCities)
 
for city1 = 1 to numCities by 1 do
    for city2 = 1 to numCities by 1 do      
	--Find dist between cities
	xD = cityCoords[city1][1] - cityCoords[city2][1]
	yD = cityCoords[city1][2] - cityCoords[city2][2]
	cityDistances[city1][city2] = sqrt(xD*xD + yD*yD)
    end for
end for
end procedure
 
----------------------------------------------------------
--Initializations
procedure setup()
    atom angle    --Only used in the circular test case
    integer ir, p1, p2, p4, lengt, numCitiesCubed
numCitiesCubed = numCities * numCities
cityCoords  = repeat(repeat(0,2),numCities)
 
--Load cityCoords with random x and y coordinates. Replace this with
--real coordinates for any specific problem you have.
for j = 1 to numCities by 1 do   --400 wide, 400 long, 150 to right
	cityCoords[j][1] =  1.01*(150 + rand(400))   --100 down
	cityCoords[j][2] =  1.01*(100 + rand(400))  --1.01 to make sure
						 --it works with floats
end for
------------------------------------------
--The following 5 lines can be uncommented to set the coords to a circle
--for test purposes as given in the introductory comments. The circular
--plotted path and right length is taken by me to indicate a converged
--correct answer.
 
--for j = 1 to numCities by 1 do        --gives right distance
    --angle = 2*pi * (j / numCities)
    --cityCoords[j][1] =  (1+sin(angle))  * 200
    --cityCoords[j][2] =  (1+cos(angle))  * 200
--end for
------------------------------------------
--The following 6 lines can be uncommented to set the coords to a polygon,
--whose length is calculatable for checking returned distance,
--for test purposes as given in the introductory comments. The distance 
--found is correct...at least for 4 sides.
 
--p1 = 200 p2 = 200   p3 = 50 p4 = -50   --gives right distance
--for j = 1 to numCities by 1 do
    --p3 =  p3   p4 = (-1) * p4
    --cityCoords[j][1] =  p1 + p3 * j
    --cityCoords[j][2] =  p1 + p4 * j
--end for   --length should be 
 
-------------------------------------------
--Now populate the distances sequence
loadDistances()
 
--Now initialize the oldPath sequence, first in 1..numCities order, then
--randomize it for an unbiased start.
for i = 1 to numCities by 1 do  oldTrip[i] = i      end for
--now mix them up a bit
for i = 1 to numCities by 1 do
ir = rand(numCities)
if ir != i then
p1 = oldTrip[i]
p2 = oldTrip[ir]
oldTrip[ir] = p1
oldTrip[i] = p2
end if
end for
end procedure
 
-------------------------------------------------------------
procedure setIndexes()  --Called by anneal to help manage the perturbation
	
    --I will define the insertion process as removing a segment, combining 
    --the 2 remanents, and then inserting the removed part at a random index
    --in the combined 2 remenants. The result will be newTrip.
    -- SI (the first term of the removal segment),
    --  EI   (the last  term of the removal segment),
    --  II   (the first term of the new end remanent)
    --for instance if {1,2,3,4,5,6,7,8,9} is changed to
	    --a remanant of {1,2,3,7,8,9} 
	    --and then to {1,2,3,7,8,4,5,6,9}  then
    --SI was 4, EI was 6, and II was 6 (the index
    --for the 9 in 1,2,3,7,8,9) 
    SI  = rand(numCities - 1)  
    EI    = rand(numCities - SI) + SI -1    --The extra calls seem to improve
    EI    = rand(numCities - SI) + SI -1    --convergence sometimes??
    IL = EI - SI + 1                
    II = rand(numCities - IL)
    II = rand(numCities - IL)
end procedure
----------------------------------------------------------------------
 
procedure anneal()  
--This does the perturbing, saves the improvements, and issues progress data
atom    oldDist, newDist, distDiff
atom fact         --fact used only for user info re value of (n-1!
sequence segment  --The removed segment to be (maybe reversed and) reinserted
integer count, reverseFlag, halfThere
if remainder(runLimit,2) = 0 then halfThere = runLimit/2 
else halfThere = (runLimit + 1) / 2  
end if
count = 0
oldDist = calcTripDist(oldTrip)
for i = 1 to runLimit  do    
  newDist = oldDist
  reverseFlag = rand(2)    --reverse the segment half the time
  setIndexes()        --random indicees for segment removal and re-insertion
  newTrip = oldTrip[1..SI - 1] & oldTrip[(EI +1)..numCities]  
    --newTrip is a truncated trip with a segment removed
    -- SI (index for the first term of the removal segment),
    --  EI   (index for the last  term of the removal segment),
    --  II   (index for the first term of the new end remanent)
    --Now calc  "change" in Distance
if (SI - 1) < 1 then city1 = oldTrip[numCities]
else  city1 = oldTrip[SI-1] 
end if
  city2 = oldTrip[EI+1] 
  newDist = newDist +   cityDistances[city1][city2] 
    --^ compensated for added leg at oldTrip SI when head and tail are combined
if (SI - 1) < 1 then city1 = oldTrip[numCities]
  else  city1 = oldTrip[SI-1] 
end if
  city2 = oldTrip[SI] 
  newDist = newDist -   cityDistances[city1][city2] 
    --^ compensated for removed leg at oldTrip SI
  city1 = oldTrip[EI]   city2 = oldTrip[EI+1] 
  newDist = newDist -   cityDistances[city1][city2] 
    --^ compensated for removed leg at oldTrip EI and EI+1
  segment = oldTrip[SI..EI]
  if reverseFlag = 2 then segment = reverse(segment) end if
  if II-1 < 1 then    city1 = newTrip[length(newTrip)] 
  else city1 = newTrip[II-1]
  end if 
  city2 = newTrip[II] 
  newDist = newDist -   cityDistances[city1][city2] 
    --^ compensated for removed legs from truncated newTrip at II-1 and II
  newTrip = newTrip[1..II-1] & segment
	& newTrip[II..(numCities - IL)]
    -- newTrip is now full length
  if II-1 < 1 then    city1 = newTrip[length(newTrip)] 
   else city1 = newTrip[II-1]
  end if
  city2 = newTrip[II] 
  newDist = newDist +   cityDistances[city1][city2] 
    --^ compensated for inserted leg at II-1 and II
  city1 = newTrip[II-1 + IL]  city2 = newTrip[II+IL]
  newDist = newDist +   cityDistances[city1][city2] 
    --^ compensated for inserted leg at II-1 + length(segment)
  if newDist < oldDist then 
    if remainder(i,10) = 2 and i < halfThere and (oldDist - newDist) > 0.00001 
    then   printf(1, "%8.2f       ", newDist)    ? i 
    elsif oldDist - newDist > 0.00001 and i > halfThere then 
      printf(1, "%8.2f       ", newDist)         ? i 
    end if
    oldTrip = newTrip
    oldDist = newDist
    count = i
  end if
end for
printf(1, "%8.2f       ", oldDist)   ? count
fact = 1  for i = 1 to numCities-1 by 1 do    fact = i * fact   end for
puts(1,"^Trip length    ^ ^iter #^\n")
puts(1,"\nLast iterations is #: ") ? runLimit
puts(1,"nNumber of Cities is: ") ? numCities
puts(1,"factorial(numCities-1) is:\n ") printf(1,"%g", fact)
puts(1,"\n Press Enter to Quit.")
end procedure
 
------------------------------------------------------------------------------
--MAIN program
atom t
object o
t = time()
if not select_mode(GRAPHICS_MODE) then
    puts(1, "COULDN'T FIND A GOOD GRAPHICS MODE\n")   abort(1) end if
puts(1,"Input an integer for the number of cities. I will give them\n")
puts(1,"random coordinates and show my approx for the shortest path\n")
numCities = getn()
runLimit = numCities * numCities * numCities
setup()
anneal()
puts(1,"\n Elapsed time was. ")   ? (time() - t)
drawIt()
--Reset the graphic mode as we exit
o = getc(0) ? o if graphics_mode(-1) then end if
 
 
 
